<?php declare(strict_types=1);

namespace Everyware\Widget\ArticleList;

use Everyware\ProjectPlugin\Components\ComponentSettingsRepository;
use Everyware\ProjectPlugin\Components\Contracts\Admin;
use Everyware\ProjectPlugin\Components\Contracts\InfoManager;
use Everyware\ProjectPlugin\Components\SettingsProviders\SimpleSettingsProvider;
use Everyware\ProjectPlugin\Components\WidgetAdmin;
use Everyware\ProjectPlugin\Helpers\FileReader;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Support\Environment;
use Infomaker\Everyware\Support\Str;
use Everyware\ProjectPlugin\Components\Adapters\WidgetAdapter;
use Infomaker\Everyware\Twig\View;
use OcAPI;

/**
 * Class ArticleListWidget
 */
class ArticleListWidget extends WidgetAdapter
{

    /**
     * The setting fields that the widget dependents on
     * @var array
     */
    protected static $fields = [
        'title' => '',
        'page' => '',
        'template' => '',
        'render_as_list' => false,
        'oc_query' => '',
        'oc_query_start' => 1,
        'oc_query_limit' => 10,
        'oc_query_sort' => '',
    ];

    /**
     * @var OcAPI
     */
    private $ocApi;

    /**
     * @var string
     */
    private $defaultTemplate = 'ocarticle-default.php';

    /**
     * @var InfoManager
     */
    protected static $infoManager;



    /**
     * Convert the form-data into values readable by OpenContent::search()
     *
     * @param $viewData
     *
     * @return array
     */
    protected function convertWidgetQuery($viewData): array
    {
        $query = $viewData['oc_query'];

        if ( ! Str::contains($query, 'contenttype:Article')) {
            $viewData['oc_query'] = QueryBuilder::query($query)->prepend('contenttype:Article')->getQuery();
        }

        return $viewData;
    }

    /**
     * Function to fetch the articles from Open Content
     *
     * @param $viewData
     *
     * @return array
     */
    private function fetchArticles($viewData): array
    {
        // Subtract oc_query_start with 1 since we do not start on 0 in admin form.
        if ((int) $viewData['oc_query_start'] > 0) {
            $viewData['oc_query_start'] = $viewData['oc_query_start'] - 1;
        }

        $this->ocApi->prepare_widget_query($this->convertWidgetQuery($viewData));

        return $this->generateFromTemplate($this->ocApi->get_oc_articles(), $viewData['template']);
    }

    /**
     * Run the article through the template to generate HTML
     *
     * @param \OcArticle[] $ocArticles
     * @param string       $template
     *
     * @return array
     */
    private function generateFromTemplate($ocArticles, $template = ''): array
    {
        $templateFile = Str::notEmpty($template) ? $template : $this->defaultTemplate;
        $template = locate_template([$templateFile, "/templates/{$templateFile}"]);

        $articles = [];
        ob_start();
        foreach ($ocArticles as $index => $article) {
            $templateFile = $template;
            include $templateFile;
            $articles[] = ob_get_contents();
            ob_clean();
        }
        ob_end_clean();

        return $articles;
    }

    /**
     * @param array $viewData
     * @param array $args
     *
     * @return string
     */
    protected function generateWidget(array $viewData, array $args): string
    {
        return View::generate('@widgets/widget', array_replace($viewData, [
            'args' => $args,
            'articles' => $this->fetchArticles($viewData),
            'settings' => $this->getWidgetSettings()
        ]));
    }

    protected function widgetSetup(): Admin
    {
        $this->ocApi = new OcAPI();

        return new WidgetAdmin(
            new ArticleListForm(static::$infoManager ?? new FileReader(__FILE__), $this->ocApi),
            new ComponentSettingsRepository(new SimpleSettingsProvider(static::$fields))
        );
    }

    public static function setInfoManager(InfoManager $infoManager): void
    {
        static::$infoManager = $infoManager;
    }
}
